import os
import json

from .utils import create_folder
from .utils import create_file
from .utils import remove_folder
from .utils import remove_file
from .utils import change_dir
from .utils import wite_to_file
from .utils import _getDirs


def create_dir(d):
    """A Recrusive Fuction which creates folders and files.
    
    Parameters:
    ===========
    d : a dict object which contains 
        structure of files and folder.
    """
    tp_dirs = list(d.keys())
    for dir in tp_dirs:
        if type(d[dir]) == type({}):
            create_folder(dir)
            change_dir(dir)
            create_dir(d[dir])
        elif d[dir] == None:
            create_file(dir)
    change_dir('../')


def remove_dir(d):
    """A Recrusive Fuction which remove folders and files.
    
    Parameters:
    ===========
    d : a dict object which contains 
        structure of files and folder.
    """
    tp_dirs = list(d.keys())
    for dir in tp_dirs:
        if type(d[dir]) == type({}):
            change_dir(dir)
            remove_dir(d[dir])
            remove_folder(dir)
        elif d[dir] == None:
            remove_file(dir)
    change_dir('../')


def create_Alldirs():
    """A Fuction which creates folders and files."""
    f = open(f'{os.getcwd()}\\patl.json', 'r')
    d = json.load(f)
    create_dir(d)
    f.close()
    print("Patl> All folders and files are created successfully.")


def remove_Alldirs():
    """A Fuction which removes folders and files."""
    f = open(f'{os.getcwd()}\\patl.json', 'r')
    d = json.load(f)
    remove_dir(d)
    f.close()
    print("Patl> All folders and files are removed successfully.")


def getdirs(l, j=None, d=None, fresh=True, ignored = []):
    """A Recrusive Fuction which scans folders and files.
    
    Parameters:
    ===========
    l : a list of dirs
    j : old key goes here in second iter.
    d : old dict goes here in second iter.
    fresh : To Tell wheter recrusive is first time or not.
    """
    if fresh:
        d = {}
    else:
        os.chdir(j)
        a = {}
    for i in l:
        print(i, )
        if fresh:
            dir = _getDirs(i)
            if i not in ignored:
                d[i] = dir
                if dir:
                    getdirs(dir, i, d, False, ignored)
                    os.chdir('../')
        else:
            dir = _getDirs(i)
            if i not in ignored:
                a[i] = dir
                d[j] = a
                if dir:
                    getdirs(dir, i, a, False, ignored)
                    os.chdir('../')
    return d

def scan_Alldirs():
    """A Fuction which scans folders and files."""
    ignore_this = [z.strip('/\n') for z in open('.patlignore', 'r').readlines()]
    lis = [u for u in os.listdir() if u not in ignore_this]
    d = getdirs(lis, ignored=ignore_this)
    wite_to_file(d)
    print("Patl> All folders and files are scanned successfully to patl.json.")





import argparse

from .core import create_Alldirs
from .core import remove_Alldirs
from .core import scan_Alldirs


parser = argparse.ArgumentParser()

parser.add_argument('-c', '--create', action='store_true', help="To create files and folders as per patl.json file says")

parser.add_argument('-r', '--remove', action='store_true', help="To remove files and folders as per patl.json file says" )

parser.add_argument('-s', '--scan', action='store_true', help="To scan files and folders into patl.json file." )

def main():
    args = parser.parse_args()

    if args.create:
        create_Alldirs()

    if args.remove:
        remove_Alldirs()

    if args.scan:
        scan_Alldirs()


import os
import json

def create_folder(file):
    try:
        os.mkdir(file)
    except Exception:
        pass

def create_file(file):
    try:
        open(file, 'w')
    except Exception:
        pass

def remove_folder(file):
    try:
        os.rmdir(file)
    except Exception:
        pass

def remove_file(file):
    try:
        os.remove(file)
    except Exception:
        pass

def change_dir(dir):
    try:
        os.chdir(dir)
    except Exception:
        pass

def _getDirs(name):
    try:
        a = os.listdir(name)
    except NotADirectoryError:
        a = None

    return a

def wite_to_file(d):
    with open('patl.json', 'w', encoding="utf-8") as f:
        json.dump(d, f, indent=4, sort_keys=True)


from setuptools import setup, find_packages
import patl

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="Patl",
    version=patl.version,
    author="Ajay Patil",
    author_email="ajay.patil.a01@gmail.com",
    description="Patl helps you in creating a monitorable structure for your files and folder of your big project.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Ajay1290/patl",
    license='MIT License',
    packages=find_packages(),
    entry_points = {
        'console_scripts': ['patl=patl.commands:main'],
    }
)

